/****************************************************************************
 * @file     main.c
 * @version  V3.00
 * $Revision: 2 $
 * $Date: 15/04/07 11:05a $
 * @brief
 *           Show the usage of QEI compare function.
 *
 * @note
 * Copyright (C) 2014 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "NM1530.h"

#define PLLCON_SETTING  CLK_PLLCON_72MHz_HXT
#define PLL_CLOCK       72000000


#define QEI0A   P80
#define QEI0B   P81


/*---------------------------------------------------------------------------------------------------------*/
/* Define functions prototype                                                                              */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main(void);

/*---------------------------------------------------------------------------------------------------------*/
/*  QEI0 IRQ Handler                                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
void QEI0_IRQHandler(void)
{
    if(QEI0->STATUS & QEI_STATUS_CMPF_Msk)      /* Compare-match flag */
    {
        printf("Compare-match INT!\n\n");
        QEI0->STATUS = QEI_STATUS_CMPF_Msk;
    }

    if(QEI0->STATUS & QEI_STATUS_OVUNF_Msk)      /* Counter Overflow or underflow flag */
    {
        printf("Overflow INT!\n\n");
        QEI0->STATUS = QEI_STATUS_OVUNF_Msk;
    }
}

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Enable Internal RC 22.1184MHz clock */
    CLK->PWRCON |= CLK_PWRCON_OSC22M_EN_Msk;

    /* Waiting for Internal RC clock ready */
    while(!(CLK->CLKSTATUS & CLK_CLKSTATUS_OSC22M_STB_Msk));

    /* Switch HCLK clock source to Internal RC and HCLK source divide 1 */
    CLK->CLKSEL0 &= ~CLK_CLKSEL0_HCLK_S_Msk;
    CLK->CLKSEL0 |= CLK_CLKSEL0_HCLK_S_HIRC;
    CLK->CLKDIV &= ~CLK_CLKDIV_HCLK_N_Msk;
    CLK->CLKDIV |= CLK_CLKDIV_HCLK(1);

    /* Enable external XTAL 12MHz clock */
    CLK->PWRCON |= CLK_PWRCON_XTL12M_EN_Msk;

    /* Waiting for external XTAL clock ready */
    while(!(CLK->CLKSTATUS & CLK_CLKSTATUS_XTL12M_STB_Msk));

    /* Set core clock as PLL_CLOCK from PLL */
    CLK->PLLCON = PLLCON_SETTING;
    while(!(CLK->CLKSTATUS & CLK_CLKSTATUS_PLL_STB_Msk));
    CLK->CLKSEL0 &= (~CLK_CLKSEL0_HCLK_S_Msk);
    CLK->CLKSEL0 |= CLK_CLKSEL0_HCLK_S_PLL;

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    //SystemCoreClockUpdate();
    PllClock        = PLL_CLOCK;            // PLL
    SystemCoreClock = PLL_CLOCK / 1;        // HCLK
    CyclesPerUs     = PLL_CLOCK / 1000000;  // For SYS_SysTickDelay()

    /* Enable UART0 and QEI0 module clock */
    CLK->APBCLK |= (CLK_APBCLK_UART0_EN_Msk | CLK_APBCLK_QEI0_EN_Msk);

    /* Select UART module clock source */
    CLK->CLKSEL1 &= ~CLK_CLKSEL1_UART_S_Msk;
    CLK->CLKSEL1 |= CLK_CLKSEL1_UART_S_HXT;

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Set P3 multi-function pins for UART0 RXD and TXD */
    SYS->P3_MFP &= ~(SYS_MFP_P30_Msk | SYS_MFP_P31_Msk);
    SYS->P3_MFP |= (SYS_MFP_P30_UART0_RXD | SYS_MFP_P31_UART0_TXD);

    /* Set P2 multi-function pins for QEI0_A, QEI0_B, QEI0_Z */
    SYS->P2_MFP &= ~(SYS_MFP_P24_Msk | SYS_MFP_P25_Msk | SYS_MFP_P26_Msk);
    SYS->P2_MFP |= (SYS_MFP_P24_QEI0_A | SYS_MFP_P25_QEI0_B | SYS_MFP_P26_QEI0_Z);

}

void UART0_Init()
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset UART0  */
    SYS->IPRSTC2 |=  SYS_IPRSTC2_UART0_RST_Msk;
    SYS->IPRSTC2 &= ~SYS_IPRSTC2_UART0_RST_Msk;

    /* Configure UART0 and set UART0 Baudrate */
    UART0->BAUD = UART_BAUD_MODE2 | UART_BAUD_MODE2_DIVIDER(__HXT, 115200);
    UART0->LCR = UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_1;
}

/*---------------------------------------------------------------------------------------------------------*/
/* MAIN function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main(void)
{

    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Init System, peripheral clock and multi-function I/O */
    SYS_Init();

    /* Lock protected registers */
    SYS_LockReg();

    /* Init UART0 for printf */
    UART0_Init();

    printf("\n\nCPU @ %dHz\n", SystemCoreClock);

    printf("+--------------------------------------+\n");
    printf("|    NM1530 QEI Driver Sample Code     |\n");
    printf("+--------------------------------------+\n\n");
    printf("  >> Please connect P8.0 and P2.4 << \n");
    printf("  >> Please connect P8.1 and P2.5 << \n");
    printf("     Press any key to start test\n\n");
    getchar();


    /* Configure P8.0 and P8.1 as output mode */
    P8->PMD = P8->PMD & (~GPIO_PMD_PMD0_Msk) | (GPIO_PMD_OUTPUT << GPIO_PMD_PMD0_Pos);
    P8->PMD = P8->PMD & (~GPIO_PMD_PMD1_Msk) | (GPIO_PMD_OUTPUT << GPIO_PMD_PMD1_Pos);

    QEI0A = 0;
    QEI0B = 0;

    /* Reset counter value */
    QEI0->CNT = 0;

    /* Set counter compare value */
    QEI0->CNTCMP = 0x10000;

    /* Set counter maximum value */
    QEI0->CNTMAX = 0x20000;

    /* Set QEI function configuration */
    QEI0->CTL |= QEI_CTL_QEIEN_Msk |        /* Enable QEI controller function */
                 QEI_CTL_CMPEN_Msk |        /* Enable compare function */
                 QEI_CTL_CMPIEN_Msk |       /* Enable compare function interrupt */
                 QEI_CTL_OVUNIEN_Msk |      /* Enable overflow function interrupt */
                 QEI_CTL_X4_COMPARE_COUNTING_MODE | /* Set QEI counting mode as X4 Compare-counting mode */
                 QEI_CTL_CHAEN_Msk |        /* Enable IDX, QEA and QEB input to QEI controller */
                 QEI_CTL_CHBEN_Msk |
                 QEI_CTL_IDXEN_Msk;

    /* Enable QEI0 IRQ */
    NVIC_EnableIRQ(QEI0_IRQn);

    /* Wait compare-match and overflow interrupt happened */
    while(1)
    {
        QEI0A = 1;
        CLK_SysTickDelay(16);
        QEI0B = 1;
        CLK_SysTickDelay(16);
        QEI0A = 0;
        CLK_SysTickDelay(16);
        QEI0B = 0;
        CLK_SysTickDelay(16);
    }

}
